<?php
/* --------------------------------------------------------------
   GroupDescriptions.php 2020-10-21
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2020 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

declare(strict_types=1);

namespace Gambio\Admin\Modules\AccessGroup\Model\ValueObjects;

use Webmozart\Assert\Assert;

/**
 * Class AccessGroupDescriptions
 *
 * @package Gambio\Admin\Modules\AccessGroup\Model\ValueObjects
 */
class AccessGroupDescriptions
{
    /**
     * @var array<int, string>
     */
    private $descriptions;
    
    
    /**
     * AccessGroupDescriptions constructor.
     */
    private function __construct()
    {
        $this->descriptions = [];
    }
    
    
    /**
     * @param array $descriptions
     *
     * @return AccessGroupDescriptions
     */
    public static function create(array $descriptions): AccessGroupDescriptions
    {
        $collection = new self();
        foreach ($descriptions as $languageId => $name) {
            $collection->addDescription($languageId, $name);
        }
        
        return $collection;
    }
    
    
    /**
     * @param int $languageId
     *
     * @return string
     */
    public function getDescription(int $languageId): string
    {
        return $this->descriptions[$languageId] ?? '';
    }
    
    
    /**
     * @param int    $languageId
     * @param string $description
     */
    public function addDescription(int $languageId, string $description): void
    {
        Assert::greaterThan($languageId, 0, 'Language ID need to be greater than 0. Got: %s');
        Assert::notWhitespaceOnly($description, 'Description can not be empty.');
        
        $this->descriptions[$languageId] = $description;
    }
}