<?php
/* --------------------------------------------------------------
   AccessRoleNames.php 2020-10-20
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2020 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

declare(strict_types=1);

namespace Gambio\Admin\Modules\AccessRole\Model\ValueObjects;

use Webmozart\Assert\Assert;

/**
 * Class AccessRoleNames
 *
 * @package Gambio\Admin\Modules\AccessRole\Model\ValueObjects
 */
class AccessRoleNames
{
    /**
     * @var array<int, string>
     */
    private $names;
    
    
    /**
     * AccessRoleNames constructor.
     */
    private function __construct()
    {
        $this->names = [];
    }
    
    
    /**
     * @param array $names
     *
     * @return AccessRoleNames
     */
    public static function create(array $names): AccessRoleNames
    {
        $collection = new self();
        foreach ($names as $languageId => $name) {
            $collection->addName($languageId, $name);
        }
        
        return $collection;
    }
    
    
    /**
     * @param int $languageId
     *
     * @return string
     */
    public function getName(int $languageId): string
    {
        return $this->names[$languageId] ?? '';
    }
    
    
    /**
     * @param int    $languageId
     * @param string $name
     */
    public function addName(int $languageId, string $name): void
    {
        Assert::greaterThan($languageId, 0, 'Language ID need to be greater than 0. Got: %s');
        Assert::notWhitespaceOnly($name, 'Role name can not be empty.');
        
        $this->names[$languageId] = $name;
    }
}