<?php
/* --------------------------------------------------------------
   WorkaroundAuthenticator.php 2021-07-22
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2021 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

declare(strict_types=1);

namespace Gambio\Api\Application\Auth;

use Gambio\Api\Application\Auth\Interfaces\WebRequestAuthenticator;
use Gambio\Core\Auth\Exceptions\AuthenticationException;
use Gambio\Core\Auth\UserId;
use Gambio\Core\Cache\Services\CacheFactory;
use Gambio\Core\Cache\Services\SafeCache;
use Psr\Http\Message\ServerRequestInterface;

/**
 * Class WorkaroundAuthenticator
 *
 * @package Gambio\Core\Auth
 */
class WorkaroundAuthenticator implements WebRequestAuthenticator
{
    /**
     * @var SafeCache
     */
    private $cache;
    
    
    /**
     * WorkaroundAuthenticator constructor.
     *
     * @param CacheFactory $cacheFactory
     */
    public function __construct(CacheFactory $cacheFactory)
    {
        $this->cache = $cacheFactory->createCacheFor('api-workaround-auth');
    }
    
    
    /**
     * @inheritDoc
     */
    public function authenticateWebRequest(ServerRequestInterface $request): UserId
    {
        $token = $request->getHeader('X-Api-Workaround-Auth');
        if (isset($token[0]) === false) {
            throw new AuthenticationException('No API workaround auth token provided.');
        }
        
        $userId = $this->cache->get(md5($token[0]));
        if ($userId === null) {
            throw new AuthenticationException('Invalid API workaround auth token provided.');
        }
        
        return \Gambio\Core\Auth\Model\UserId::create($userId);
    }
}