<?php
/* --------------------------------------------------------------
   WorkaroundAuthenticatorTrait.php 2021-07-22
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2021 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

declare(strict_types=1);

namespace Gambio\Api\Application\Auth;

use Gambio\Core\Cache\Services\CacheFactory;
use Gambio\Core\Cache\Services\SafeCache;

trait WorkaroundAuthenticatorTrait
{
    /**
     * @var SafeCache
     */
    private $workaroundAuthCache;
    
    /**
     * @var int
     */
    private $workaroundAuthUserId;
    
    
    /**
     * @param CacheFactory $cacheFactory
     * @param int          $userId
     */
    private function initWorkaroundAuth(CacheFactory $cacheFactory, int $userId)
    {
        $this->workaroundAuthCache  = $cacheFactory->createCacheFor('api-workaround-auth');
        $this->workaroundAuthUserId = $userId;
    }
    
    
    /**
     * @return string
     */
    private function getWorkaroundAuthToken(): string
    {
        // Create new token
        $token = uniqid('', true);
        
        // Replace token if user already got a token
        $index = $this->workaroundAuthCache->get('index', []);
        if (isset($index[$this->workaroundAuthUserId])) {
            $token = $index[$this->workaroundAuthUserId];
        }
        
        // Update index
        $index[$this->workaroundAuthUserId] = $token;
        
        // Cache index and token
        $this->workaroundAuthCache->set('index', $index);
        $this->workaroundAuthCache->set(md5($token), $this->workaroundAuthUserId);
        
        return $token;
    }
}