<?php
/* --------------------------------------------------------------
   dependent.inc.php 2021-12-14
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2021 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------*/

use Gambio\StyleEdit\Core\Services\Configuration\ConfigurationService;
use Gambio\StyleEdit\Core\Services\StyleEdit3Configuration\Entities\Interfaces\StyleEdit3ConfigurationInterface;
use Gambio\StyleEdit\Core\Services\StyleEdit3Configuration\StyleEdit3ConfigurationService;
use Gambio\StyleEdit\Core\SingletonPrototype;
use Gambio\StyleEdit\DependencyInjector;

/** @var DatabaseModel $this */
/** @var bool $t_success */

// check if Honeygrid template is active
$currentTemplate = $this->query('SELECT `value` FROM `gx_configurations` WHERE `key` = "configuration/CURRENT_TEMPLATE"');
$currentTemplate = isset($currentTemplate[0]) ? (string)$currentTemplate[0]['value'] : '';

$currentTheme = $this->query('SELECT `value` FROM `gx_configurations` WHERE `key` = "configuration/CURRENT_THEME"');
$currentTheme = isset($currentTheme[0]) ? (string)$currentTheme[0]['value'] : '';

if ($currentTemplate !== '' && $currentTheme === '' && file_exists(DIR_FS_CATALOG . 'StyleEdit3')) {
    try {
        if ($currentTemplate === 'Honeygrid') {
            $honegridStyleEdit3ConfigurationFilePath = '';
        
            DependencyInjector::inject();
        
            /** @var StyleEdit3ConfigurationService $styleEdit3ConfigurationService */
            $styleEdit3ConfigurationService = SingletonPrototype::instance()->get(StyleEdit3ConfigurationService::class);
        
            $configurations = $styleEdit3ConfigurationService->configurations();
        
            /** @var StyleEdit3ConfigurationInterface $configuration */
            foreach ($configurations as $configuration) {
                if ($configuration->type() === 'Template' && $configuration->name() === 'Honeygrid'
                    && $configuration->isActive()) {
                    $honegridStyleEdit3ConfigurationFilePath = $configuration->path();
                }
            }
        
            if ($honegridStyleEdit3ConfigurationFilePath !== '') {
                $styleEdit3Configuration = $styleEdit3ConfigurationService->getConfigurationByPath($honegridStyleEdit3ConfigurationFilePath);
                $configuration           = SingletonPrototype::instance()
                    ->get(ConfigurationService::class)
                    ->convertFromStyleEdit3($styleEdit3Configuration);
            
                // convert configuration to array
                $configuration = json_decode(json_encode($configuration), true);
            
                // copy image files to images directory, because template directory will be deleted and update image path accordingly
                foreach ($configuration as &$config) {
                    if (is_string($config['value']) && strpos($config['value'], 'templates/Honeygrid') !== false) {
                        $oldImagePath = substr($config['value'], strpos($config['value'], 'templates/Honeygrid'));
                        $newImagePath = str_replace('templates/Honeygrid/assets/images/custom',
                                                    'images/themes/Honeygrid',
                                                    $oldImagePath);
                    
                        $newImageDirectoryPath = DIR_FS_CATALOG . dirname($newImagePath);
                        if (!is_dir($newImageDirectoryPath)) {
                            mkdir($newImageDirectoryPath, 0777, true);
                        
                            // recursive permissions do not work so set them one by one
                            chmod($newImageDirectoryPath, 0777);
                            chmod(dirname($newImageDirectoryPath), 0777);
                        }
                    
                        $config['value'] = $newImagePath;
                    
                        if (file_exists(DIR_FS_CATALOG . $oldImagePath) && !file_exists(DIR_FS_CATALOG . $newImagePath)) {
                            copy(DIR_FS_CATALOG . $oldImagePath, DIR_FS_CATALOG . $newImagePath);
                            chmod(DIR_FS_CATALOG . $newImagePath, 0777);
                        }
                    }
                }
            
                // sort the configuration by the "name" value and put the "custom-scss-css-style" configuration to the end
                usort($configuration,
                    function ($a, $b) {
                        if ($a['name'] === 'custom-scss-css-style') {
                            return 1;
                        }
                        if ($b['name'] === 'custom-scss-css-style') {
                            return -1;
                        }
                    
                        return $a['name'] <=> $b['name'];
                    });
            
                $json = json_encode($configuration, JSON_PRETTY_PRINT);
            
                $honeygridThemeSettingsFilePath = DIR_FS_CATALOG . 'themes/Honeygrid/settings.json';
            
                // create backup of existing settings.json
                if (file_exists($honeygridThemeSettingsFilePath)) {
                    rename($honeygridThemeSettingsFilePath,
                           dirname($honeygridThemeSettingsFilePath) . '/settings.backup-' . date('Ymd_His') . '.json');
                }
            
                // ensure themes/Honeygrid directory is writable
                @chmod(dirname($honeygridThemeSettingsFilePath), 0777);
            
                if (is_writable(dirname($honeygridThemeSettingsFilePath))) {
                    // write new settings.json
                    file_put_contents($honeygridThemeSettingsFilePath, $json);
                }
            }
        }
    } catch (Throwable $e) {
        // skip Honeygrid style migration, if shop environment cannot be booted without errors
    }
    
    // activate Honeygrid theme
    $query     = 'UPDATE `gx_configurations` SET `value` = "Honeygrid" WHERE `key` = "configuration/CURRENT_THEME"';
    $t_success &= is_numeric($this->query($query));
}

$check = $this->table_column_exists('orders', 'exported');
if (!$check) {
    $query     = "ALTER TABLE `orders` ADD `exported` TINYINT(4) NOT NULL DEFAULT '0' COMMENT 'already exported to WaWi?' AFTER `orders_hash`";
    $t_success &= $this->query($query);
}

if (!file_exists(DIR_FS_CATALOG . 'config/.env.php')) {
    $tokenFiles = glob(DIR_FS_CATALOG . 'media/secure_token_*');
    
    $securityToken = bin2hex(random_bytes(16));
    if (isset($tokenFiles[0])) {
        $securityToken = (string)str_replace('secure_token_', '', basename($tokenFiles[0]));
    }
    
    $sentryDsn = '';
    $check     = $this->query('SELECT * FROM `gx_configurations`
                            WHERE
                                `key` = "gm_configuration/is_cloud" AND
                                `value` = "true"',
                              true);
    if ($check->num_rows > 0) {
        $sentryDsn = 'https://2206275f8e974f719ca49e33b3ca05e4:400277ba99c44ab58edda99ac097345e@telemetry.gambio-server.net/9';
    }
    
    $envFilePath    = DIR_FS_CATALOG . 'cache/.env.php';
    $envFileContent = <<<ENV
<?php

return [
    'APP_SECURITY_TOKEN' => '%s',
    'APP_SENTRY_DSN'     => '%s',
];
ENV;
    
    file_put_contents($envFilePath, sprintf($envFileContent, $securityToken, $sentryDsn));
    @chmod($envFilePath, 0777);
}

$checkField = $this->query("DESCRIBE `orders` 'gm_orders_id'", true);
if ($checkField->num_rows > 0) {
    $t_query   = "ALTER TABLE `orders` DROP `gm_orders_id`";
    $t_success &= $this->query($t_query);
}

$checkField = $this->query("DESCRIBE `orders` 'gm_packings_id'", true);
if ($checkField->num_rows > 0) {
    $t_query   = "ALTER TABLE `orders` DROP `gm_packings_id`";
    $t_success &= $this->query($t_query);
}

$checkField = $this->query("DESCRIBE `orders` 'gm_orders_code'", true);
if ($checkField->num_rows > 0) {
    $t_query   = "ALTER TABLE `orders` DROP `gm_orders_code`";
    $t_success &= $this->query($t_query);
}

$checkField = $this->query("DESCRIBE `orders` 'gm_packings_code'", true);
if ($checkField->num_rows > 0) {
    $t_query   = "ALTER TABLE `orders` DROP `gm_packings_code`";
    $t_success &= $this->query($t_query);
}

$check = $this->query("SELECT * FROM `gx_configurations` WHERE `key` = 'gm_configuration/SHOW_NORMAL_PRICE_ONLY_FOR_NATIONAL_SHIPPING'",
                      true);
if ($check->num_rows < 1) {
    $query     = "INSERT INTO gx_configurations  (`key`, `value`, `default`, `type`, `sort_order`) VALUES ('gm_configuration/SHOW_NORMAL_PRICE_ONLY_FOR_NATIONAL_SHIPPING', 'false', 'false', 'switcher', NULL)";
    $t_success &= $this->query($query, true);
}

$check = $this->query("SELECT * FROM `gx_configurations` WHERE `key` = 'gm_configuration/DEFAULT_TAX_CLASS_ID'", true);
if ($check->num_rows < 1) {
    $query     = "INSERT INTO gx_configurations  (`key`, `value`, `default`, `type`, `sort_order`) VALUES ('gm_configuration/DEFAULT_TAX_CLASS_ID', '1', '1', 'tax-class', NULL)";
    $t_success &= $this->query($query, true);
}

$t_success &= $this->set_index('products',
                               'INDEX',
                               ['products_status', 'products_startpage'],
                               'products_status_products_startpage');

$check = $this->query("SELECT * FROM `gx_configurations` WHERE `key` = 'gm_configuration/GM_PASSWORD_ENCRYPTION_TYPE' AND `value` = 'md5'",
                      true);
if ($check->num_rows === 1) {
    $query     = 'UPDATE `gx_configurations` SET `value` = "password_hash" WHERE `key` = "gm_configuration/GM_PASSWORD_ENCRYPTION_TYPE"';
    $t_success &= is_numeric($this->query($query));
    
    $query     = 'UPDATE `gx_configurations` SET `value` = "true" WHERE `key` = "gm_configuration/GM_PASSWORD_REENCRYPT"';
    $t_success &= is_numeric($this->query($query));
}

if ($this->columnExists('products_options', 'options_id') === false) {
    
    $query = "
        ALTER TABLE `products_options`
            ADD `options_id` INT NULL DEFAULT NULL COMMENT 'mirrors `properties`.`properties_id`' AFTER `language_id`;";
    
    $t_success &= $this->query($query, true);
}

if ($this->columnExists('products_properties_combis', 'gtin') === false) {
    
    $query = "
        ALTER TABLE `products_properties_combis`
            ADD `gtin` VARCHAR(64) NULL DEFAULT NULL AFTER `vpe_value`;";
    
    $t_success &= $this->query($query, true);
}

if ($this->columnExists('products_properties_combis', 'asin') === false) {
    
    $query = "
        ALTER TABLE `products_properties_combis`
            ADD `asin` VARCHAR(64) NULL DEFAULT NULL AFTER `gtin`;";
    
    $t_success &= $this->query($query, true);
}

if ($this->columnExists('products_properties_combis', 'stock_type') === false) {
    
    $query = "
        ALTER TABLE `products_properties_combis`
            ADD `stock_type` varchar(16) NULL DEFAULT 'not-managed' AFTER `combi_ean`;";
    
    $t_success &= $this->query($query, true);
}

if ($this->columnExists('products_properties_combis', 'combi_weight_type') === false) {
    
    $query = "
        ALTER TABLE `products_properties_combis`
            ADD `combi_weight_type` enum('calc','fix') NULL DEFAULT NULL AFTER `combi_weight`;";
    
    $t_success &= $this->query($query, true);
}

if ($this->columnExists('products_properties_combis', 'products_vpe_id')) {
    
    $query     = "ALTER TABLE `products_properties_combis` CHANGE `products_vpe_id` `products_vpe_id` INT(11) NULL DEFAULT '0';";
    $t_success &= $this->query($query, true);
}

/**
 * ADD NEW COLUMNS TO products_attributes
 */

if ($this->columnExists('products_attributes', 'stock_type') === false) {
    
    $query     = "ALTER TABLE `products_attributes` ADD `stock_type` varchar(16) NULL DEFAULT 'not-managed' AFTER `price_prefix`;";
    $t_success &= $this->query($query, true);
}

/**
 * ADD NEW COLUMNS TO properties_description
 */

if ($this->columnExists('properties_description', 'description') === false) {
    $query     = "ALTER TABLE `properties_description` ADD `description` varchar(255) NOT NULL DEFAULT '' AFTER `properties_admin_name`;";
    $t_success &= $this->query($query, true);
}

/**
 * UPDATE EXISTING COLUMNS IN properties_values
 */

if ($this->columnExists('properties_values', 'display_image') === false) {
    $query     = "UPDATE `properties_values` SET `display_image` = '' WHERE `display_image` IS NULL;";
    $t_success &= $this->query($query, true);
    
    $query     = "ALTER TABLE `properties_values` CHANGE COLUMN `display_image` `display_image` VARCHAR(256) NOT NULL DEFAULT '';";
    $t_success &= $this->query($query, true);
}

/**
 * ADD NEW COLUMNS TO properties_values
 */

if ($this->columnExists('properties_values', 'weight') === false) {
    $query     = "ALTER TABLE `properties_values` ADD `weight` decimal(9,4) NOT NULL DEFAULT '0.0000' AFTER `value_model`;";
    $t_success &= $this->query($query, true);
}

if ($this->columnExists('properties_values', 'stock_type') === false) {
    $query     = "ALTER TABLE `properties_values` ADD `stock_type` VARCHAR(32) NOT NULL DEFAULT 'not-managed' AFTER `display_image`;";
    $t_success &= $this->query($query, true);
}

if ($this->columnExists('properties_values', 'stock') === false) {
    $query     = "ALTER TABLE `properties_values` ADD `stock` decimal(9,4) NOT NULL DEFAULT 0 AFTER `display_image`;";
    $t_success &= $this->query($query, true);
}

if ($this->columnExists('properties_values', 'stock_centrally_managed') === false) {
    $query     = "ALTER TABLE `properties_values` ADD `stock_centrally_managed` int(1) NOT NULL DEFAULT 0 AFTER `display_image`;";
    $t_success &= $this->query($query, true);
}

/**
 * ADD NEW COLUMNS TO properties_values_description
 */

if ($this->columnExists('properties_values_description', 'description') === false) {
    $query     = "ALTER TABLE `properties_values_description` ADD `description` varchar(255) NOT NULL DEFAULT '' AFTER `values_name`;";
    $t_success &= $this->query($query, true);
}

/**
 * ADD NEW COLUMNS TO products_options
 */

if ($this->columnExists('products_options', 'admin_label') === false) {
    $query     = "ALTER TABLE `products_options` ADD `admin_label` varchar(255) NOT NULL DEFAULT '' COMMENT 'mirrors `properties_description`.`properties_admin_name`' AFTER `products_option_display_type`;";
    $t_success &= $this->query($query, true);
}

if ($this->columnExists('products_options', 'description') === false) {
    $query     = "ALTER TABLE `products_options` ADD `description` varchar(255) NOT NULL DEFAULT '' COMMENT 'mirrors `properties_description`.`description`' AFTER `products_option_display_type`;";
    $t_success &= $this->query($query, true);
}

if ($this->columnExists('products_options', 'sort_order') === false) {
    $query     = "ALTER TABLE `products_options` ADD `sort_order` INT NULL DEFAULT NULL COMMENT 'mirrors `properties`.`sort_order`' AFTER `products_option_display_type`;";
    $t_success &= $this->query($query, true);
}

/**
 * ADD NEW COLUMNS TO products_options_values
 */

if ($this->columnExists('products_options_values', 'option_value_id') === false) {
    $query     = "ALTER TABLE `products_options_values` ADD `option_value_id` INT NULL DEFAULT NULL COMMENT 'mirrors `properties_values`.`properties_values_id`' AFTER `gm_filename`;";
    $t_success &= $this->query($query, true);
}

if ($this->columnExists('products_options_values', 'description') === false) {
    $query     = "ALTER TABLE `products_options_values` ADD `description` varchar(255) NOT NULL DEFAULT '' COMMENT 'mirrors `properties_values_description`.`description`' AFTER `gm_filename`;";
    $t_success &= $this->query($query, true);
}

if ($this->columnExists('products_options_values', 'sort_order') === false) {
    $query     = "ALTER TABLE `products_options_values` ADD `sort_order` int(10) unsigned NOT NULL DEFAULT '0' COMMENT 'mirrors `properties_values`.`sort_order`' AFTER `gm_filename`;";
    $t_success &= $this->query($query, true);
}

if ($this->columnExists('products_options_values', 'value_model') === false) {
    $query     = "ALTER TABLE `products_options_values` ADD `value_model` varchar(64) NOT NULL DEFAULT '' COMMENT 'mirrors `properties_values`.`value_model`' AFTER `gm_filename`;";
    $t_success &= $this->query($query, true);
}

if ($this->columnExists('products_options_values', 'weight') === false) {
    $query     = "ALTER TABLE `products_options_values` ADD `weight` decimal(9,4) NOT NULL DEFAULT '0.0000' COMMENT 'mirrors `properties_values`.`weight`' AFTER `gm_filename`;";
    $t_success &= $this->query($query, true);
}

if ($this->columnExists('products_options_values', 'value_price') === false) {
    $query     = "ALTER TABLE `products_options_values` ADD `value_price` decimal(9,4) NOT NULL DEFAULT '0.0000' COMMENT 'mirrors `properties_values`.`value_price`' AFTER `gm_filename`;";
    $t_success &= $this->query($query, true);
}

if ($this->columnExists('products_options_values', 'display_image') === false) {
    $query     = "ALTER TABLE `products_options_values` ADD `display_image` VARCHAR(256) NULL COMMENT 'mirrors `properties_values`.`display_image`' AFTER `gm_filename`;";
    $t_success &= $this->query($query, true);
}

if ($this->columnExists('products_options_values', 'stock_type') === false) {
    $query     = "ALTER TABLE `products_options_values` ADD `stock_type` VARCHAR(32) NOT NULL DEFAULT 'not-managed' COMMENT 'mirrors `properties_values`.`stock_type`' AFTER `gm_filename`;";
    $t_success &= $this->query($query, true);
}

if ($this->columnExists('products_options_values', 'stock') === false) {
    $query     = "ALTER TABLE `products_options_values` ADD `stock` decimal(9,4) NOT NULL DEFAULT 0.0000 COMMENT 'mirrors `properties_values`.`stock`' AFTER `gm_filename`;";
    $t_success &= $this->query($query, true);
}

if ($this->columnExists('products_options_values', 'stock_centrally_managed') === false) {
    $query     = "ALTER TABLE `products_options_values` ADD `stock_centrally_managed` int(1) NOT NULL DEFAULT 0 COMMENT 'mirrors `properties_values`.`stock_centrally_managed`' AFTER `gm_filename`;";
    $t_success &= $this->query($query, true);
}

if ($this->columnExists('products_properties_combis_values', 'options_id') === false) {
    
    $query     = "ALTER TABLE `products_properties_combis_values` ADD `options_id` int(11) NOT NULL DEFAULT '0' COMMENT 'mirrors `properties`.`properties_id`' AFTER `properties_values_id`;";
    $t_success &= $this->query($query, true);
    $query     = "
        REPLACE INTO `products_properties_combis_values`
        (
            `products_properties_combis_values_id`,
            `products_properties_combis_id`,
            `properties_values_id`,
            `options_id`
        )
        SELECT
               `ppcv`.`products_properties_combis_values_id`,
               `ppcv`.`products_properties_combis_id`,
               `ppcv`.`properties_values_id`,
               `pv`.`properties_id`
        FROM `products_properties_combis_values` AS `ppcv`
        INNER JOIN `properties_values` AS `pv`
        ON `pv`.properties_values_id = `ppcv`.`properties_values_id`";
    $t_success &= $this->query($query, true);
}

/**
 * ADD NEW COLUMNS TO products_options_values_to_products_options
 */

if ($this->columnExists('products_options_values_to_products_options', 'options_id') === false) {
    $query     = "ALTER TABLE `products_options_values_to_products_options` ADD `options_id` int(11) NOT NULL DEFAULT '0' COMMENT 'mirrors `properties`.`properties_id`' AFTER `products_options_values_id`;";
    $t_success &= $this->query($query, true);
}

if ($this->columnExists('products_options_values_to_products_options', 'option_value_id') === false) {
    $query     = "ALTER TABLE `products_options_values_to_products_options` ADD `option_value_id` int(11) NOT NULL DEFAULT '0' COMMENT 'mirrors `properties_values`.`properties_values_id`' AFTER `products_options_values_id`;";
    $t_success &= $this->query($query, true);
}

include __DIR__ . '/product_variants_migration.php';
include __DIR__ . '/gx_lang_configurations_migration.php';


if (($groupId = $this->getAdminAccessGroupIdByIdentifier('AJAX_HANDLER', 'XMLConnect')) !== false) {
    
    $t_success &= $this->query('DELETE FROM `admin_access_groups` WHERE `admin_access_group_id` = ' . $groupId, true);
    $t_success &= $this->query('DELETE FROM `admin_access_group_descriptions` WHERE `admin_access_group_id` = '
                               . $groupId,
                               true);
    $t_success &= $this->query('DELETE FROM `admin_access_group_items` WHERE `admin_access_group_id` = ' . $groupId,
                               true);
    $t_success &= $this->query('DELETE FROM `admin_access_permissions` WHERE `admin_access_group_id` = ' . $groupId,
                               true);
}

// manipulate old Hub Connector to not throw Exception in GX 4.5
$hubServiceProviderFilePath = DIR_FS_CATALOG . 'GXModules/Gambio/Hub/Admin/Module/HubServiceProvider.php';
if (file_exists($hubServiceProviderFilePath)) {
    $fileContent = file_get_contents($hubServiceProviderFilePath);
    
    if (strpos($fileContent, 'use Gambio\Core\Application\Modules\Abstraction\AbstractModuleServiceProvider;')
        !== false) {
        $fileContent = str_replace('use Gambio\Core\Application\Modules\Abstraction\AbstractModuleServiceProvider;',
                                   'use Gambio\Core\Application\DependencyInjection\AbstractModuleServiceProvider;',
                                   $fileContent);
        unlink($hubServiceProviderFilePath);
        file_put_contents($hubServiceProviderFilePath, $fileContent);
        @chmod($fileContent, 0777);
    }
}
$hubFooterBadgeLoaderFilePath = DIR_FS_CATALOG . 'GXModules/Gambio/Hub/Admin/Module/HubFooterBadgeLoader.php';
if (file_exists($hubFooterBadgeLoaderFilePath)) {
    $fileContent = file_get_contents($hubFooterBadgeLoaderFilePath);
    
    if (strpos($fileContent, "class_exists('\Gambio\Core\Application\Modules\App\DependencyInjection\AbstractModuleBootableServiceProvider')")
        !== false) {
        $fileContent = str_replace("class_exists('\Gambio\Core\Application\Modules\App\DependencyInjection\AbstractModuleBootableServiceProvider')",
                                   "class_exists('\Gambio\Core\Application\DependencyInjection\AbstractModuleBootableServiceProvider')",
                                   $fileContent);
        unlink($hubFooterBadgeLoaderFilePath);
        file_put_contents($hubFooterBadgeLoaderFilePath, $fileContent);
        @chmod($fileContent, 0777);
    }
}
$hubTranslationsLoaderFilePath = DIR_FS_CATALOG . 'GXModules/Gambio/Hub/Admin/Module/HubTranslationsLoader.php';
if (file_exists($hubTranslationsLoaderFilePath)) {
    $fileContent = file_get_contents($hubTranslationsLoaderFilePath);
    
    if (strpos($fileContent, "class_exists('\Gambio\Core\TextManager\Services\TextManager')")
        !== false) {
        $fileContent = str_replace("class_exists('\Gambio\Core\TextManager\Services\TextManager')",
                                   "interface_exists('\Gambio\Core\TextManager\Services\TextManager')",
                                   $fileContent);
        unlink($hubTranslationsLoaderFilePath);
        file_put_contents($hubTranslationsLoaderFilePath, $fileContent);
        @chmod($fileContent, 0777);
    }
}

// #68720
$result = $this->query('SELECT * FROM `gx_configurations` WHERE `key` = "configuration/SESSION_KEEP_ALIVE"', true);
if ((int)$result->num_rows <= 0) {
    $this->query(<<<MYSQL
INSERT INTO
    `gx_configurations` (`key`, `value`, `default`, `type`, `sort_order`)
    VALUES ("configuration/SESSION_KEEP_ALIVE", "1", "1", "switcher", "9");
MYSQL
    );
}


// removing skrill from list of installed payment modules
$installedPaymentModules = $this->getConfiguration('configuration/MODULE_PAYMENT_INSTALLED');
$installedPaymentModules = explode(';', $installedPaymentModules);
$installedPaymentModules = array_filter($installedPaymentModules, static function(string $module): bool {
    return preg_match('/^skrill/', $module) !== 1;
});
$installedPaymentModules = implode(';', $installedPaymentModules);
$this->replaceConfiguration('configuration/MODULE_PAYMENT_INSTALLED', $installedPaymentModules);


if ($this->table_exists('download_options')) {
    
    $t_success &= $this->query('DROP TABLE `download_options`', true);
}

$invoiceConfigurationQuery = "
        SELECT * FROM `gx_configurations`
        WHERE `key` = 'gm_configuration/GM_INVOICE_ID' AND `value` LIKE 'R_{DELIVERY_ID}_%'
    ";

$invoiceResult = $this->query($invoiceConfigurationQuery, true);

if ((int)$invoiceResult->num_rows === 1) {
    
    $t_success &= $this->query("UPDATE `gx_configurations` SET `value` = CONCAT('R_{INVOICE_ID}_', YEAR(NOW())) WHERE `key` = 'gm_configuration/GM_INVOICE_ID'", true);
}

/**
 * ADDING ADDITIONAL COLUMNS TO PARCEL SERVICE AND TRACKING CODES FOR
 * - is-return-delivery flag
 * - shipment type
 */

if ($this->columnExists('parcel_services', 'shipment_type') === false) {
    $query     = "ALTER TABLE `parcel_services` ADD `shipment_type` VARCHAR(255) NOT NULL DEFAULT '' AFTER `default`";
    
    $t_success &= $this->query($query, true);
}

if ($this->columnExists('orders_parcel_tracking_codes', 'is_return_delivery') === false) {
    $query     = "ALTER TABLE `orders_parcel_tracking_codes` ADD `is_return_delivery` INT(1) UNSIGNED NOT NULL DEFAULT '0' AFTER `tracking_code`";
    
    $t_success &= $this->query($query, true);
}

if ($this->columnExists('orders_parcel_tracking_codes', 'shipment_type') === false) {
    $query     = "ALTER TABLE `orders_parcel_tracking_codes` ADD `shipment_type` VARCHAR(255) NOT NULL DEFAULT '' AFTER `comment`";
    
    $t_success &= $this->query($query, true);
}

if ($this->table_exists('google_export_condition') === false) {
    
    $createExportConditionIdQuery = "
            CREATE TABLE `google_export_condition`
            (
                `google_export_condition_id` INT(10)     AUTO_INCREMENT,
                `language_id`                INT         DEFAULT 2 NOT NULL,
                `condition`                  varchar(64) NOT NULL DEFAULT '',
                PRIMARY KEY (`google_export_condition_id`, `language_id`)
            ) ENGINE = InnoDB
              DEFAULT CHARSET = `utf8`;";
    
    $t_success &= $this->query($createExportConditionIdQuery, true);
    
    $addExportConditionRowsQuery = "
        INSERT INTO `google_export_condition` (`google_export_condition_id`, `language_id`, `condition`)
        VALUES (1, 2, 'neu'),
               (2, 2, 'gebraucht'),
               (3, 2, 'erneuert'),
               (1, 1, 'new'),
               (2, 1, 'used'),
               (3, 1, 'renewed');";
    
    $t_success &= $this->query($addExportConditionRowsQuery, true);
}

if ($this->columnExists('products_item_codes', 'google_export_condition_id') === false) {
    $query     = "ALTER TABLE `products_item_codes` ADD `google_export_condition_id` INT(10) NOT NULL DEFAULT '1' AFTER `code_jan`;";
    $t_success &= $this->query($query, true);
}

if ($this->columnExists('products_item_codes', 'google_export_condition')) {
    $exportConditions = [
        1 => 'neu',
        2 => 'gebraucht',
        3 => 'erneuert'
    ];
    
    $query = 'UPDATE `products_item_codes` SET `google_export_condition_id` = %d WHERE `google_export_condition` = "%s"';
    
    foreach ($exportConditions as $googleExportConditionId => $condition) {
        
        $t_success &= $this->query(sprintf($query, $googleExportConditionId, $condition), true);
    }
    
    $query     = "ALTER TABLE `products_item_codes` DROP `google_export_condition`;";
    $t_success &= $this->query($query, true);
}
