<?php
/*--------------------------------------------------------------
   gx_lang_configurations_migration.php 2021-12-13
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2021 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
 -------------------------------------------------------------*/

/** @var DatabaseModel $this */
/** @var bool $t_success */

if ($this->columnExists('gx_configurations', 'language_id')) {
    
    $t_success &= $this->query('DROP TABLE IF EXISTS `gx_lang_configurations`');
    $createTableQuery = '
        CREATE TABLE IF NOT EXISTS `gx_lang_configurations` (
            `id`            int          NOT NULL PRIMARY KEY AUTO_INCREMENT,
            `key`           varchar(255) NOT NULL,
            `language_id`   int          NOT NULL,
            `value`         mediumtext,
            `default`       mediumtext,
            `sort_order`    int          DEFAULT NULL,
            `last_modified` timestamp    NULL DEFAULT CURRENT_TIMESTAMP,
            UNIQUE INDEX `key_language_id`(`key`, `language_id`)
        )
            ENGINE = InnoDB
            DEFAULT CHARSET = `utf8`
    ';
    
    $t_success &= $this->query($createTableQuery, true);
    
    $languageDependentConfigurations = [
        'gm_configuration/EMAIL_BILLING_SUBJECT_ORDER',
        'gm_configuration/GM_FOOTER',
        'gm_configuration/GM_PDF_COMPANY_ADRESS_LEFT',
        'gm_configuration/GM_PDF_COMPANY_ADRESS_RIGHT',
        'gm_configuration/GM_PDF_CONDITIONS',
        'gm_configuration/GM_PDF_EMAIL_SUBJECT',
        'gm_configuration/GM_PDF_FOOTER_CELL_1',
        'gm_configuration/GM_PDF_FOOTER_CELL_2',
        'gm_configuration/GM_PDF_FOOTER_CELL_3',
        'gm_configuration/GM_PDF_FOOTER_CELL_4',
        'gm_configuration/GM_PDF_HEADING_CANCELLATION_INVOICE',
        'gm_configuration/GM_PDF_HEADING_CONDITIONS',
        'gm_configuration/GM_PDF_HEADING_INFO_TEXT_INVOICE',
        'gm_configuration/GM_PDF_HEADING_INFO_TEXT_PACKINGSLIP',
        'gm_configuration/GM_PDF_HEADING_INVOICE',
        'gm_configuration/GM_PDF_HEADING_PACKINGSLIP',
        'gm_configuration/GM_PDF_HEADING_WITHDRAWAL',
        'gm_configuration/GM_PDF_INFO_TEXT_INVOICE',
        'gm_configuration/GM_PDF_INFO_TEXT_PACKINGSLIP',
        'gm_configuration/GM_PDF_INFO_TITLE_INVOICE',
        'gm_configuration/GM_PDF_INFO_TITLE_PACKINGSLIP',
        'gm_configuration/GM_PDF_INVOICES_EMAIL_SUBJECT',
        'gm_configuration/GM_PDF_SALUTATION_FEMALE',
        'gm_configuration/GM_PDF_SALUTATION_MALE',
        'gm_configuration/GM_PDF_WITHDRAWAL',
        'gm_configuration/GM_TITLE_STANDARD_META_TITLE',
        'gm_configuration/GM_TITLE_STANDARD_META_TITLE_SEPARATOR',
        'gm_configuration/keywords',
        'gm_configuration/robots',
    ];
    
    $languageDependentConfigurationKeys = array_map(static function(string $key): string {
        return sprintf('"%s"', $key);
    }, $languageDependentConfigurations);
    $languageDependentConfigurationKeys = implode(', ', $languageDependentConfigurationKeys);
    
    $insertQuery = '
        INSERT INTO `gx_lang_configurations` (`key`, `language_id`, `value`, `default`, `sort_order`)
        SELECT `key`, `language_id`, `value`, `default`, `sort_order`
        FROM `gx_configurations`
        WHERE `key` IN (%s)';
    
    $t_success &= $this->query(sprintf($insertQuery, $languageDependentConfigurationKeys), true);
    
    $removeDuplicateConfigurationsQuery = 'DELETE FROM `gx_configurations` WHERE `key` IN (%s)';
    $t_success &= $this->query(sprintf($removeDuplicateConfigurationsQuery, $languageDependentConfigurationKeys), true);
    
    /**
     * all meta tags now are stored with the key prefix "gm_configuration/meta/"
     */
    $renameMetaTagKeysQuery = "UPDATE `gx_lang_configurations` SET `key` = REPLACE(`key`, 'gm_configuration/', 'gm_configuration/meta/') WHERE `key` IN ('gm_configuration/robots', 'gm_configuration/keywords');";
    $t_success &= $this->query($renameMetaTagKeysQuery, true);
    
    /**
     * changing old meta tag keys
     */
    $renameMetaTagKeysQuery = "UPDATE `gx_configurations` SET `key` = REPLACE(`key`, 'gm_configuration/', 'gm_configuration/meta/') WHERE `legacy_group_id` = 9999;";
    $t_success &= $this->query($renameMetaTagKeysQuery, true);
    
    /**
     * migrating old meta tags to gx_lang_configurations
     */
    $insertMetaTagsQuery = '
        INSERT INTO `gx_lang_configurations` (`key`, `language_id`, `value`, `default`, `sort_order`)
        SELECT `key`, `language_id`, `value`, `default`, `sort_order`
        FROM `gx_configurations`
        WHERE `key` LIKE "gm_configuration/meta/%"';
    $t_success &= $this->query($insertMetaTagsQuery, true);
    
    /**
     * deleting meta configurations in "gx_configurations" table
     */
    $deleteMetaTagsFromGxConfiguration = 'DELETE FROM `gx_configurations` WHERE `key` LIKE "gm_configuration/meta/%";';
    $t_success &= $this->query($deleteMetaTagsFromGxConfiguration, true);
    
    $this->drop_index('gx_configurations', 'uc_cfg_key');
    $this->drop_index('gx_configurations', 'key');
    $this->set_index('gx_configurations', 'UNIQUE', ['key'], 'key');
    
    $removeLanguageColumnQuery = 'ALTER TABLE `gx_configurations` DROP `language_id`';
    $t_success &= $this->query($removeLanguageColumnQuery, true);
    
    $removeLanguageColumnQuery = 'ALTER TABLE `gx_configurations` DROP `legacy_group_id`';
    $t_success &= $this->query($removeLanguageColumnQuery, true);
}